"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parse_1 = require("./parse");
/** kebab regex */
const kr = /[A-Z\u00C0-\u00D6\u00D8-\u00DE]/g;
/** kebab case */
function kc(pattern) {
    return pattern.replace(kr, match => '-' + match.toLowerCase());
}
/** upwrap property */
/** TODO: refine it */
function up(prop, indent = '  ') {
    if (typeof prop === 'object') {
        return (' {\n' +
            Object.entries(prop).map(v => indent + `  ${kc(v[0])}: ${v[1]};`).join('\n') +
            '\n' + indent + '}');
    }
    return `: ${String(prop)};`;
}
/** create style */
function cs(selector, props, instance) {
    if (props === null) {
        return null;
    }
    const propertyNames = Object.keys(props);
    if (propertyNames.length === 0) {
        if (instance.config.preserveEmptyBlock)
            return selector + ' {}';
        return null;
    }
    const statements = [
        selector + ' {'
    ];
    propertyNames.forEach(propertyName => {
        const property = props[propertyName];
        const unwrappedProperty = typeof property === 'function' ? property() : property;
        propertyName = kc(propertyName);
        statements.push(`  ${propertyName}${up(unwrappedProperty)}`);
    });
    statements.push('}');
    return statements.join('\n');
}
/** traverse */
function t(node, paths, styles, instance) {
    const pathIsString = typeof node.path === 'string';
    if (pathIsString)
        paths.push(node.path);
    else {
        if (node.path.before !== undefined)
            node.path.before(instance.context);
        paths.push(node.path.$(instance.context));
    }
    const selector = parse_1.p$p(paths, instance);
    const style = cs(selector, node.props, instance);
    if (style !== null)
        styles.push(style);
    if (node.children !== null) {
        node.children.forEach(childNode => {
            t(childNode, paths, styles, instance);
        });
    }
    paths.pop();
    if (!pathIsString && node.path.after !== undefined)
        node.path.after(instance.context);
}
function render(node, instance) {
    const styles = [];
    t(node, [], styles, instance);
    return styles.join('\n\n');
}
exports.render = render;
